@@ -1,6 +1,11 @@
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import unicode_literals
+
 import functools
-from functools import update_wrapper, partial
-import inspect, itertools
+from functools import partial
+import inspect
+import itertools
 import logging
 import time
 from traceback import format_exc
@@ -8,9 +13,12 @@ import sys
 import warnings
 
 import wrapt
+import six
+from six.moves import range, map
 
 log = logging.getLogger(__name__)
 
+
 def format_argspec_plus(fn, grouped=True):
     """Returns a dictionary of formatted, introspected function arguments.
 
@@ -43,6 +51,7 @@ def format_argspec_plus(fn, grouped=True):
        'apply_pos': '(self, a, b, c, **d)'}
 
     """
+    warnings.warn('format_argspec_plus is deprecated and will be removed.', DeprecationWarning, 2)
     spec = callable(fn) and inspect.getargspec(fn) or fn
     args = inspect.formatargspec(*spec)
     if spec[0]:
@@ -62,12 +71,11 @@ def format_argspec_plus(fn, grouped=True):
         return dict(args=args[1:-1], self_arg=self_arg,
                     apply_pos=apply_pos[1:-1], apply_kw=apply_kw[1:-1])
 
+
 def unique_symbols(used, *bases):
     used = set(used)
     for base in bases:
-        pool = itertools.chain((base,),
-                               itertools.imap(lambda i: base + str(i),
-                                              xrange(1000)))
+        pool = itertools.chain((base,), map(lambda i: base + str(i), range(1000)))
         for sym in pool:
             if sym not in used:
                 used.add(sym)
@@ -76,23 +84,14 @@ def unique_symbols(used, *bases):
         else:
             raise NameError("exhausted namespace for symbol base %s" % base)
 
+
 def decorator(target):
-    """A signature-matching decorator factory."""
+    warnings.warn('decorator is deprecated and will be removed. Use wrapt.', DeprecationWarning, 2)
 
-    def decorate(fn):
-        spec = inspect.getargspec(fn)
-        names = tuple(spec[0]) + spec[1:3] + (fn.func_name,)
-        targ_name, fn_name = unique_symbols(names, 'target', 'fn')
-
-        metadata = dict(target=targ_name, fn=fn_name)
-        metadata.update(format_argspec_plus(spec, grouped=False))
-
-        code = 'lambda %(args)s: %(target)s(%(fn)s, %(apply_kw)s)' % (
-                metadata)
-        decorated = eval(code, {targ_name:target, fn_name:fn})
-        decorated.func_defaults = getattr(fn, 'im_func', fn).func_defaults
-        return update_wrapper(decorated, fn)
-    return update_wrapper(decorate, target)
+    @wrapt.decorator
+    def wrapper(wrapped, instance, args, kwargs):
+        return target(wrapped, *args, **kwargs)
+    return wrapper
 
 
 def _num_required_args(func):
@@ -113,7 +112,7 @@ def _num_required_args(func):
         borrowed from: https://github.com/pytoolz/toolz
     """
     try:
-        spec = inspect.getargspec(func)
+        spec = inspect.getfullargspec(func)
         if spec.varargs:
             return None
         num_defaults = len(spec.defaults) if spec.defaults else 0
@@ -162,7 +161,7 @@ class curry(object):
         self.keywords = kwargs if kwargs else None
         self.__doc__ = self.func.__doc__
         try:
-            self.func_name = self.func.func_name
+            self.__name__ = self.func.__name__
         except AttributeError:
             pass
 
@@ -208,11 +207,12 @@ def deprecate(message):
         Decorate a function to emit a deprecation warning with the given
         message.
     """
-    @decorator
-    def decorate(fn, *args, **kw):
+    @wrapt.decorator
+    def wrapper(wrapped, instance, args, kwargs):
         warnings.warn(message, DeprecationWarning, 2)
-        return fn(*args, **kw)
-    return decorate
+        return wrapped(*args, **kwargs)
+    return wrapper
+
 
 def exc_emailer(send_mail_func, logger=None, catch=Exception, print_to_stderr=True):
     """
@@ -232,32 +232,34 @@ def exc_emailer(send_mail_func, logger=None, catch=Exception, print_to_stderr=Tr
     if logger is None:
         logger = log
 
-    @decorator
-    def decorate(fn, *args, **kwargs):
+    @wrapt.decorator
+    def wrapper(wrapped, instance, args, kwargs):
         exc_info = None
         try:
-            return fn(*args, **kwargs)
-        except catch, e:
+            return wrapped(*args, **kwargs)
+        except catch as e:
             body = format_exc()
             exc_info = sys.exc_info()
             error_msg = 'exc_mailer() caught an exception, email will be sent.'
             logger.exception(error_msg)
             if print_to_stderr:
-                print >> sys.stderr, error_msg + '  ' + str(e)
+                print(error_msg + '  ' + str(e), file=sys.stderr)
             try:
                 send_mail_func(body)
             except Exception:
-                logger.exception('exc_mailer(): send_mail_func() threw an exception, logging it & then re-raising original exception')
-                raise exc_info[0], exc_info[1], exc_info[2]
+                logger.exception('exc_mailer(): send_mail_func() threw an exception, '
+                                 'logging it & then re-raising original exception')
+                six.reraise(exc_info[0], exc_info[1], exc_info[2])
         finally:
             # delete the traceback so we don't have garbage collection issues.
             # see warning at: http://docs.python.org/library/sys.html#sys.exc_info
             if exc_info is not None:
                 del exc_info
-    return decorate
+    return wrapper
+
 
 class Retry(object):
-    def __init__(self, tries, exceptions, delay=0.1, logger=None, msg=None):
+    def __init__(self, tries, exceptions, delay=0.1, logger=None, msg=None, level=logging.DEBUG):
         """
         Decorator for retrying a function if exception occurs
 
@@ -277,24 +279,31 @@ class Retry(object):
             self.log = logger
         else:
             self.log = log
+
+        self.log_level = level
         self.msg = msg
 
     def __call__(self, fn):
         @functools.wraps(fn)
         def wrapfn(*args, **kwargs):
-            for _ in xrange(self.tries):
+            for try_count in range(self.tries):
                 try:
                     return fn(*args, **kwargs)
-                except self.exceptions, e:
+                except self.exceptions as e:
                     if self.msg is not None and self.msg not in str(e):
                         raise
-                    self.log.debug("Retry, exception: %s", e)
+
+                    message = "Retry, exception: {}".format(e)
+                    self.log.log(self.log_level, message)
+
+                    if try_count == self.tries - 1:
+                        # no tries left, reraise
+                        raise
+
                     time.sleep(self.delay)
-            # should only get to this point of we have an
-            # exception and have run out of tries
-            raise
+
         return wrapfn
-retry = Retry
+retry = Retry  # noqa: E305
 
 
 class hybrid_method(object):
@@ -379,4 +388,3 @@ def memoize(method_to_wrap):
     decorated = inner_memoize(method_to_wrap)
     decorated.reset_memoize = reset_memoize
     return decorated
-
