@@ -1,11 +1,19 @@
+from __future__ import absolute_import
+from __future__ import unicode_literals
+
 import csv
 import difflib
 from pprint import PrettyPrinter
+import six
 import time
 
 from blazeutils.datastructures import OrderedDict
+from blazeutils.sentinels import NotGivenIter
 
-def tolist(x, default=[]):
+
+def tolist(x, default=NotGivenIter):
+    if default is NotGivenIter:
+        default = []
     if x is None:
         return default
     if not isinstance(x, (list, tuple)):
@@ -13,7 +21,10 @@ def tolist(x, default=[]):
     else:
         return x
 
-def ensure_list(x, default=[]):
+
+def ensure_list(x, default=NotGivenIter):
+    if default is NotGivenIter:
+        default = []
     if x is None:
         return default
     if isinstance(x, tuple):
@@ -23,6 +34,7 @@ def ensure_list(x, default=[]):
     else:
         return x
 
+
 def ensure_tuple(x):
     if x is None:
         return ()
@@ -30,6 +42,7 @@ def ensure_tuple(x):
         return x
     return (x,)
 
+
 def toset(x):
     if x is None:
         return set()
@@ -38,16 +51,19 @@ def toset(x):
     else:
         return x
 
-def pprint( stuff, indent = 4):
+
+def pprint(stuff, indent=4):
     pp = PrettyPrinter(indent=indent)
     pp.pprint(stuff)
 
-def pformat(stuff, indent = 4):
+
+def pformat(stuff, indent=4):
     pp = PrettyPrinter(indent=indent)
     return pp.pformat(stuff)
 
+
 def is_iterable(possible_iterable):
-    if isinstance(possible_iterable, basestring):
+    if isinstance(possible_iterable, six.string_types):
         return False
     try:
         iter(possible_iterable)
@@ -55,20 +71,23 @@ def is_iterable(possible_iterable):
     except TypeError:
         return False
 
+
 def is_empty(value):
     """ a boolean test except 0 and False are considered True """
-    if not value and value is not 0 and value is not False:
+    if not value and value != 0 and value is not False:
         return True
     return False
 
+
 def multi_pop(d, *args):
     """ pops multiple keys off a dict like object """
     retval = {}
     for key in args:
-        if d.has_key(key):
+        if key in d:
             retval[key] = d.pop(key)
     return retval
 
+
 def grouper(records, *fields):
     grouped_records = OrderedDict()
 
@@ -83,11 +102,11 @@ def grouper(records, *fields):
         final_kpos = len(location)-1
         for kpos, key in enumerate(location):
             if kpos != final_kpos:
-                if not at.has_key(key):
+                if key not in at:
                     at[key] = OrderedDict()
                 at = at[key]
             else:
-                if not at.has_key(key):
+                if key not in at:
                     at[key] = []
                 at[key].append(record)
 
@@ -95,26 +114,29 @@ def grouper(records, *fields):
         setup_grouping(record, *fields)
     return grouped_records
 
+
 class Tee(object):
-     """A file-like that writes to all the file-likes it has."""
+    """A file-like that writes to all the file-likes it has."""
 
-     def __init__(self, *files):
-         """Make a Tee that writes to all the files in `files.`"""
-         self.files = files
+    def __init__(self, *files):
+        """Make a Tee that writes to all the files in `files.`"""
+        self.files = files
+
+    def write(self, data):
+        """Write `data` to all the files."""
+        for f in self.files:
+            f.write(data)
 
-     def write(self, data):
-         """Write `data` to all the files."""
-         for f in self.files:
-             f.write(data)
 
 def csvtolist(inputstr):
     """ converts a csv string into a list """
     reader = csv.reader([inputstr], skipinitialspace=True)
     output = []
     for r in reader:
-      output += r
+        output += r
     return output
 
+
 class Timer(object):
 
     def __init__(self):
@@ -126,6 +148,7 @@ class Timer(object):
     def elapsed(self, name='default'):
         return time.time() - self.timers[name]
 
+
 def unique(seq, preserve_order=True):
     """
         Take a sequence and make it unique.  Not preserving order is faster, but
@@ -137,19 +160,22 @@ def unique(seq, preserve_order=True):
         # f8 by Dave Kirby
         # Order preserving
         seen = set()
-        return [x for x in seq if x not in seen and not seen.add(x)]
+        seen_add = seen.add  # lookup method only once
+        return [x for x in seq if x not in seen and not seen_add(x)]
     # f9
     # Not order preserving
-    return {}.fromkeys(seq).keys()
+    return list({}.fromkeys(seq).keys())
+
 
 def prettifysql(sql):
     """Returns a prettified version of the SQL as a list of lines to help
     in creating a useful diff between two SQL statements."""
     pretty = []
     for line in sql.split('\n'):
-        pretty.extend([ "%s,\n" % x for x in line.split(',')])
+        pretty.extend(["%s,\n" % x for x in line.split(',')])
     return pretty
 
+
 def diff(actual, expected):
     """
         normalize whitespace in actual and expected and return unified diff
